/************************************************************
 Parallelport Epromsimulator bis 64kB mit Xilinx XC9572
 ************************************************************
 hk@holger-klabunde.de

 Last change: 16.11.2001
 ***********************************************************
 Compiler: Turbo C++ V3.0
 fuer Linux: GCC 3.3.3
 ***********************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifndef __linux__
#include <conio.h>
#endif

#include "Port_io.h"

#define version "V1.00"

FILE *in = NULL;
unsigned long adress; /* Adresszaehler */
unsigned char data;   /* Alle Daten */
unsigned char steuer; /* Steuerpins */

/* reicht int hier z.B. unter Linux ? */
/* Gute Frage ... da ioperm() und Konsorten alle "unsigned long" erwarten, */
/* habe ich das vorsichtshalber mal geaendert. */

unsigned long baseadr    = 0x378;   /* Portadresse Daten */
unsigned long handshake;            /* Portadresse + 1 => Handshake vom PIC */
unsigned long control;              /* Portadresse + 2 => Kontrollregister */
unsigned long offset     = 0;

/* Prototypen */
int SendByte(unsigned char, unsigned char);
void usage(void);

/* Hauptfunktion */
int main(int argc, char *argv[])
{
 int result;
 unsigned long count;

 printf("\n***********************************\n");
 printf("***    EpromSimulator %s     ***\n",version);
 printf("***    hk@holger-klabunde.de    ***\n");
 printf("***    Use at your own risk !   ***\n");
 printf("***    Last change: 12.08.2005  ***\n");
 printf("***********************************\n\n");

 /* Hilfetext ausgeben */
 if (argc < 2) /* no filename */
 {
   usage();
   return 1;
 }

 /* Eingabedatei oeffnen */
 if ((in = fopen(argv[1], "rb")) == NULL)
 {
   printf("\nFile %s not found !\n", argv[1]);
   return 1;
 }

 /* Portadresse holen */
 if (argc >= 4) /* get portadress */
 {
   if (strcmp(argv[2], "-a") == 0)
     sscanf(argv[3], "%x", (unsigned int*)&baseadr);
 }

 /* Offset holen */
 if (argc >= 6) /* get offset */
 {
   if (strcmp(argv[4], "-o") == 0)
     sscanf(argv[5], "%lx", &offset);
 }

 printf("Settings: portadress 0x%X  offset 0x%04X\n", (unsigned int)baseadr, (unsigned int)offset);

#ifdef __linux__
 /* rohen Portzugriff anfordern */
 if (ioperm(baseadr, 3, 1) == -1)
 {
   printf("Can't get raw access to port, error %u : %s\n", errno, strerror(errno));
   printf("Are you 'root' ?\n");
   return(1);
 }
#endif

 /* Adressen bestimmen */
 handshake = baseadr + 1; /* Handshakesignaladresse */
 control   = baseadr + 2; /* Steuerportadresse */

 steuer = SIMULATOR;
 STROBE_ON();             /* Startsignal fuer PIC */

 result = SendByte(ADR_LO, 0); /* Schaltung auf Reset und einmal Daten senden */
                               /* um zu sehen ob die Schaltung da ist. */
 if (result == NO_PIC)
 {
   printf("\nEpromsimulator not found at portadress 0x%X !\n", (unsigned int)baseadr);
   fclose(in);
   in = NULL;

#ifdef __linux__
    /* Portzugriff beendet */
   ioperm(baseadr, 3, 0);
#endif

   return 1;
 }

 /* Auf Reset warten, bei PIII-500MHz ca. 1s */
 for (count = 0; count < 200000; ++count)
   outportb(baseadr, 0);

 adress = offset;

 do
 {
   result = fread(&data, sizeof(unsigned char), 1, in); /* Ein Byte lesen */

   if (result > 0)
   {
     /* Adresse Lowbyte */
     SendByte(ADR_LO, (unsigned char)(adress & 0xFF));
	 /* Adresse Highbyte */
     SendByte(ADR_HI, (unsigned char)((adress >> 8) & 0xFF));
	 /* Daten anlegen und schreiben */
     SendByte(DATEN, data);

	 ++adress; /* naechste Adresse */
	}
  } while (result > 0 && adress < 0x10000);

 /* Epromsimulator auf Eprom-Mode, Schaltungs-Reset beenden */
 SendByte(SIMULATOR, 0);

 printf("Send databytes: %lu  0x%04lX\n", adress - offset, adress - offset);

 /* Eingabedatei schliessen */
 if (in)
 {
   fclose(in);
   in = NULL;
 }

#ifdef __linux__
 /* Portzugriff beendet */
 ioperm(baseadr, 3, 0);
#endif

 return 0;
}

/* Kleiner Hilfetext */
void usage(void)
{
 printf("\nUsage: EprSim filename -a portadress [-o offset]\n");
   printf("       EprSim hello.bin -a 0x378 -o 0x0\n\n");
   printf("       Defaults:\n");
   printf("       portadress 0x378\n");
   printf("       offset 0x0000\n");
   printf("\n");
}

/* Daten mit Handshake uebergeben */
int SendByte(unsigned char port, unsigned char by)
{
 unsigned long tout;
 unsigned char b;

 outportb(baseadr, by); /* Daten anlegen */

 steuer = port;
 outportb(control, steuer); /* Strobe geht hier mit auf 0 */

 /* Deshalb hier kein STROBE_OFF(); */
 /* STROBE_OFF(); */

 tout = 100000;

 /* Warten bis PIC mit BUSY=0 (gelesen als 0x80) antwortet */
 do
 {
   b = inportb(handshake);
   --tout;

   if (tout == 0) /* Zu lange gewartet */
   {
     /* Time_Out(); */
     return NO_PIC;
   }

 } while (b < 0x80);

 STROBE_ON(); /* Signal fuer PIC: Daten liegen an */

 tout = 100000;

 /* Warten bis PIC mit BUSY=1 (gelesen als 0) Daten quittiert */
 do
 {
   b = inportb(handshake);
   --tout;

   if (tout == 0) /* Zu lange gewartet */
   {
     /* Time_Out(); */
     return NO_PIC;
   }
 } while (b > 0x7F);

 return NO_ERRORS; /* Signal alles klar */
}

/* Ende */
